#from .views import LeadVisit,PatientVisit
from .views import PatientVisit
from rest_framework import serializers
from datetime import date
#from .models import LvCancel,PvCancel
from .models import PvCancel
from brvisitApp.models import PvPayment,PatientVisitSlot
############## Patient%Module #################

# class LeadVisitCustomSerializer(serializers.ModelSerializer):
#     pt_name = serializers.CharField(source="patient.fullname",default="",read_only=True)
#     pt_number_1 = serializers.CharField(source="patient.number_1",default="",read_only=True)
#     pt_number_2 = serializers.CharField(source="patient.number_2",default="",read_only=True)
#     pt_complaint = serializers.CharField(source="patient.complaint",default="",read_only=True)
#     hour_slot_name = serializers.CharField(source="hour_slot.name",default="",read_only=True)
#     pt_age = serializers.SerializerMethodField()
#     class Meta:
#         model = LeadVisit
#         fields="__all__"

#     def get_pt_age(self, obj):
#         #https://stackoverflow.com/questions/2217488/age-from-birthdate-in-python
#         birth_date = obj.patient.birth_date
#         today = date.today()
#         return today.year - birth_date.year - ((today.month, today.day) < (birth_date.month, birth_date.day))
class PatientVisitDataSerializer(serializers.ModelSerializer):
    visit_status_string = serializers.SerializerMethodField()
    amount_paid_total = serializers.SerializerMethodField()
    visit_period = serializers.SerializerMethodField()
    p_name = serializers.CharField(source="patient.fullname",default="",read_only=True)
    p_number_1 = serializers.CharField(source="patient.number_1",default="",read_only=True)
    p_number_2 = serializers.CharField(source="patient.number_2",default="",read_only=True)
    ms_name = serializers.CharField(source="medicalservice.name",default="",read_only=True)
    initial_slot_order = serializers.CharField(source="initial_slot.order",default="",read_only=True)
    slot_name = serializers.CharField(source="initial_slot.name",default="",read_only=True)
    dr_name = serializers.CharField(source="doctor.get_full_name",default="",read_only=True)
    created_by_name = serializers.CharField(source="created_by.get_full_name",default="",read_only=True)
    created_at_str = serializers.DateTimeField(source="created_at",format="%I:%M%p\n%Y-%m-%d",default="",read_only=True)
    visit_day = serializers.DateField(format="%Y-%m-%d")

    payment_method_str = serializers.CharField(source="payment_method.name",default="",read_only=True)
    invoice_amount_remaining = serializers.IntegerField(read_only=True)  # Read-only field
    invoice_id = serializers.IntegerField(read_only=True)  # Read-only field

    def get_visit_status_string(self, obj):
        return obj.get_visit_status_display()    

    class Meta:
        model = PatientVisit
        fields="__all__"
    def get_amount_paid_total(self,obj):
        return (obj.amount_paid + obj.related_payment )   

    def get_visit_period(self, obj):
        session_period = self.context.get("session_period")
        return int(obj.slots_count *  session_period )  
    
class PatientVisitCustomSerializer(serializers.ModelSerializer):
    visit_status_str = serializers.SerializerMethodField()
    pt_name = serializers.CharField(source="patient.fullname",default="",read_only=True)
    ms_name = serializers.CharField(source="medicalservice.name",default="",read_only=True)
    doctor_name = serializers.CharField(source="doctor.name",default="",read_only=True)
    ms_price = serializers.CharField(source="medicalservice.price",default="",read_only=True)
    entry_datetime = serializers.DateTimeField(format="%d/%m/%Y %H:%M",required=False)
    created_at = serializers.DateTimeField(format="%d-%m-%Y %H:%M",required=False)

    class Meta:
        model = PatientVisit
        fields="__all__"
        read_only_fields = ('visit_status_str', 'pt_name',"ms_name","ms_price","entry_datetime")

    def get_visit_status_str(self, obj):
        return obj.get_visit_status_display()

class PatientVisitReceptionSerializer(serializers.ModelSerializer):
    p_name = serializers.CharField(source="patient.fullname",default="",read_only=True)
    p_number_1 = serializers.CharField(source="patient.number_1",default="",read_only=True)
    dr_name = serializers.CharField(source="doctor.get_full_name",default="",read_only=True)
    ms_name = serializers.CharField(source="medicalservice.name",default="",read_only=True)
    #ms_name = models.CharField(source="patient.fullname",default="",read_only=True)
    #p_name = models.CharField(source="patient.fullname",default="",read_only=True)
    class Meta:
        model = PatientVisit
        #fields="__all__"
        fields=["id","p_name","p_number_1","dr_name","ms_name","rcptn_notes","is_confirmed"]

class PatientVisitSlotSerializer(serializers.ModelSerializer):
    class Meta:
        model = PatientVisitSlot
        fields="__all__"
    def to_representation(self, instance):
        return super(PatientVisitReceptionSerializer, self).to_representation(instance) 


class PvCancelSerializer(serializers.ModelSerializer):
    class Meta:
        model = PvCancel
        fields="__all__" 

class PvPaymentSerializer(serializers.ModelSerializer):
    created_by_name = serializers.CharField(source="created_by.name",default="",read_only=True)
    created_at = serializers.DateTimeField(format="%I:%M %p @ %Y-%m-%d",read_only=True)
    class Meta:
        model = PvPayment
        fields="__all__"

class VisitExtraInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PatientVisit
        fields=["notes_for_rcptn","recommended_session","sessions_min","sessions_max"]

#
# class LvCancelSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = LvCancel
#         fields="__all__" 

class PvPaymentCustomSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(source="created_by.get_full_name",default="",read_only=True)
    #creation_time = serializers.TimeField(source="created_at",format="%I:%M %p",read_only=True)
    #creation_date = serializers.DateField(source="created_at",format="%Y-%m-%d",read_only=True)
    created_at = serializers.DateTimeField(format="%I:%M %p @ %Y-%m-%d",read_only=True)
    class Meta:
        model = PvPayment
        fields="__all__"

class PvPaymentSerializer(serializers.ModelSerializer):
    class Meta:
        model = PvPayment
        fields="__all__"